@@ -29,33 +29,29 @@
 
 """
 
-from .cpu import *
-
-from .session import ClientSession
-from .state import SessionState
+from .network import ClientSession
+from .network import SessionState
 
+from .archivist import ClientArchivist
+from .facebook import ClientFacebook
 from .messenger import ClientMessenger
 from .packer import ClientMessagePacker
 from .processor import ClientMessageProcessor, ClientContentProcessorCreator
 from .terminal import Terminal
+from .checkpoint import Checkpoint
 
 
 __all__ = [
 
-    #
-    #   CPU
-    #
-    'HandshakeCommandProcessor',
-    'LoginCommandProcessor',
-    'ReceiptCommandProcessor',
-
     #
     #   Client
     #
     'ClientSession', 'SessionState',
 
-    'ClientMessenger',
+    'ClientArchivist',
+    'ClientFacebook', 'ClientMessenger',
     'ClientMessagePacker',
     'ClientMessageProcessor', 'ClientContentProcessorCreator',
     'Terminal',
+    'Checkpoint',
 ]
