@@ -24,30 +24,38 @@
 # ==============================================================================
 
 """
-    Command Processing Unites
+    Content Processing Unites
     ~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    Processors for commands
+    Processors for contents
 """
 
-from .handshake import HandshakeCommandProcessor
-from .login import LoginCommandProcessor
-from .receipt import ReceiptCommandProcessor
-
-from .history import HistoryCommandProcessor, GroupCommandProcessor
+from .group import HistoryCommandProcessor, GroupCommandProcessor
 from .grp_invite import InviteCommandProcessor
 from .grp_expel import ExpelCommandProcessor
+from .grp_join import JoinCommandProcessor
 from .grp_quit import QuitCommandProcessor
 from .grp_reset import ResetCommandProcessor
 from .grp_query import QueryCommandProcessor
+from .grp_resign import ResignCommandProcessor
+
+from .handshake import HandshakeCommandProcessor
+from .commands import LoginCommandProcessor
+
+from .creator import ClientContentProcessorCreator
 
 __all__ = [
-    'HandshakeCommandProcessor',
-    'LoginCommandProcessor',
-    'ReceiptCommandProcessor',
 
     'HistoryCommandProcessor',
     'GroupCommandProcessor',
-    'InviteCommandProcessor', 'ExpelCommandProcessor', 'QuitCommandProcessor',
+    'InviteCommandProcessor', 'ExpelCommandProcessor',
+    'JoinCommandProcessor', 'QuitCommandProcessor',
     'ResetCommandProcessor', 'QueryCommandProcessor',
+    'ResignCommandProcessor',
+
+    'HandshakeCommandProcessor',
+    'LoginCommandProcessor',
+
+    'ClientContentProcessorCreator',
+
 ]
