@@ -24,7 +24,7 @@
 # ==============================================================================
 
 from abc import ABC, abstractmethod
-from typing import List, Tuple
+from typing import Optional, Dict, List
 
 from dimsdk import ID, ReliableMessage, CipherKeyDelegate
 
@@ -32,15 +32,16 @@ from dimsdk import ID, ReliableMessage, CipherKeyDelegate
 class ReliableMessageDBI(ABC):
     """ ReliableMessage Table """
 
+    CACHE_LIMIT = 20480  # only cache last messages
+
     @abstractmethod
-    def reliable_messages(self, receiver: ID, start: int = 0, limit: int = 1024) -> Tuple[List[ReliableMessage], int]:
+    def reliable_messages(self, receiver: ID, limit: int = 1024) -> List[ReliableMessage]:
         """
         Get network messages
 
         :param receiver: actual receiver
-        :param start:    start position for loading message
-        :param limit:    max count for loading message
-        :return: partial messages and remaining count, 0 means there are all messages cached
+        :param limit:    cache limit
+        :return: last cached messages
         """
         raise NotImplemented
 
@@ -53,11 +54,25 @@ class ReliableMessageDBI(ABC):
         raise NotImplemented
 
 
+# noinspection PyAbstractClass
 class CipherKeyDBI(CipherKeyDelegate, ABC):
     """ CipherKey Table """
     pass
 
 
-class MessageDBI(ReliableMessageDBI, CipherKeyDBI, ABC):
+class GroupKeysDBI(ABC):
+    """ Group Keys Table """
+
+    @abstractmethod
+    def group_keys(self, group: ID, sender: ID) -> Optional[Dict[str, str]]:
+        raise NotImplemented
+
+    @abstractmethod
+    def save_group_keys(self, group: ID, sender: ID, keys: Dict[str, str]) -> bool:
+        raise NotImplemented
+
+
+# noinspection PyAbstractClass
+class MessageDBI(ReliableMessageDBI, CipherKeyDBI, GroupKeysDBI, ABC):
     """ Message Database """
     pass
