@@ -72,13 +72,15 @@ class LoginCommand(BaseCommand):
 
     def __init__(self, content: Dict[str, Any] = None, identifier: ID = None):
         if content is None:
-            # create with ID
-            super().__init__(cmd=self.LOGIN)
+            # 1. new command with ID
             assert identifier is not None, 'login ID should not empty'
+            cmd = self.LOGIN
+            super().__init__(cmd=cmd)
             self['ID'] = str(identifier)
         else:
-            # create with command content
-            super().__init__(content=content)
+            # 2. command info from network
+            assert identifier is None, 'params error: %s, %s' % (content, identifier)
+            super().__init__(content)
 
     #
     #   Client Info
@@ -90,7 +92,7 @@ class LoginCommand(BaseCommand):
     # Device ID
     @property
     def device(self) -> Optional[str]:
-        return self.get('device')
+        return self.get_str(key='device', default=None)
 
     @device.setter
     def device(self, value: str):
@@ -102,7 +104,7 @@ class LoginCommand(BaseCommand):
     # User Agent
     @property
     def agent(self) -> Optional[str]:
-        return self.get('agent')
+        return self.get_str(key='agent', default=None)
 
     @agent.setter
     def agent(self, value: str):
@@ -115,32 +117,39 @@ class LoginCommand(BaseCommand):
     #   Server Info
     #
     @property
-    def station(self) -> Optional[dict]:
+    def station(self) -> Optional[Dict]:
         return self.get('station')
 
     @station.setter
-    def station(self, value: Union[dict, Station]):
-        if value is None:
-            self.pop('station', None)
-        elif isinstance(value, dict):
-            self['station'] = value
+    def station(self, info: Union[Dict, Station]):
+        if isinstance(info, Station):
+            sid = info.identifier
+            if sid.is_broadcast:
+                self['station'] = {
+                    'host': info.host,
+                    'port': info.port,
+                }
+            else:
+                self['station'] = {
+                    'ID': str(sid),
+                    'host': info.host,
+                    'port': info.port,
+                }
+        elif isinstance(info, Dict):
+            self['station'] = info
         else:
-            assert isinstance(value, Station), 'station error: %s' % value
-            self['station'] = {
-                'ID': str(value.identifier),
-                'host': value.host,
-                'port': value.port,
-            }
+            assert info is None, 'station info error: %s' % info
+            self.pop('station', None)
 
     @property
-    def provider(self) -> Optional[dict]:
+    def provider(self) -> Optional[Dict]:
         return self.get('provider')
 
     @provider.setter
-    def provider(self, value: Union[dict, ServiceProvider]):
+    def provider(self, value: Union[Dict, ServiceProvider]):
         if value is None:
             self.pop('provider', None)
-        elif isinstance(value, dict):
+        elif isinstance(value, Dict):
             self['provider'] = value
         else:
             assert isinstance(value, ServiceProvider), 'SP error: %s' % value
