@@ -29,13 +29,16 @@
 # ==============================================================================
 
 import socket
+import threading
 from abc import ABC
-from typing import Generic, TypeVar, Optional, Union, List, Tuple
+from typing import Generic, TypeVar, Optional, Union, List
 
+from startrek.types import SocketAddress
+from startrek.net.state import StateOrder
 from startrek import Hub, Channel
 from startrek import Connection, ConnectionState, ActiveConnection
 from startrek import Docker, DockerDelegate
-from startrek import Arrival, StarGate
+from startrek import Arrival, StarDocker, StarGate
 
 from ..utils import Logging
 
@@ -47,11 +50,13 @@ from .ws import WSDocker
 H = TypeVar('H')
 
 
-class BaseGate(StarGate, Generic[H], ABC):
+# noinspection PyAbstractClass
+class CommonGate(StarGate, Logging, Generic[H], ABC):
 
     def __init__(self, delegate: DockerDelegate):
         super().__init__(delegate=delegate)
         self.__hub: H = None
+        self.__lock = threading.Lock()
 
     @property
     def hub(self) -> H:
@@ -64,119 +69,48 @@ class BaseGate(StarGate, Generic[H], ABC):
     #
     #   Docker
     #
-    def get_docker(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]],
-                   advance_party: List[bytes]) -> Docker:
-        docker = self._get_docker(remote=remote, local=local)
-        if docker is None:
-            hub = self.hub
-            # from startrek import Hub
-            # assert isinstance(hub, Hub)
-            conn = hub.connect(remote=remote, local=local)
-            if conn is not None:
-                docker = self._create_docker(connection=conn, advance_party=advance_party)
-                assert docker is not None, 'failed to create docker: %s, %s' % (remote, local)
-                self._set_docker(remote=remote, local=local, docker=docker)
-        return docker
 
     # Override
-    def _get_docker(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]]) -> Optional[Docker]:
+    def _get_docker(self, remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Docker]:
         return super()._get_docker(remote=remote, local=None)
 
     # Override
-    def _set_docker(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]], docker: Docker):
-        super()._set_docker(remote=remote, local=None, docker=docker)
+    def _set_docker(self, docker: Docker,
+                    remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Docker]:
+        return super()._set_docker(docker=docker, remote=remote, local=None)
 
     # Override
-    def _remove_docker(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]], docker: Optional[Docker]):
-        super()._remove_docker(remote=remote, local=None, docker=docker)
+    def _remove_docker(self, docker: Optional[Docker],
+                       remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Docker]:
+        return super()._remove_docker(docker=docker, remote=remote, local=None)
 
-    # Override
-    def _heartbeat(self, connection: Connection):
-        # let the client to do the job
-        if isinstance(connection, ActiveConnection):
-            super()._heartbeat(connection=connection)
-
-    # Override
-    def _cache_advance_party(self, data: bytes, connection: Connection) -> List[bytes]:
-        # TODO: cache the advance party before decide which docker to use
-        if data is None or len(data) == 0:
-            return []
-        else:
-            return [data]
-
-    # Override
-    def _clear_advance_party(self, connection: Connection):
-        # TODO: remove advance party for this connection
-        pass
-
-
-class CommonGate(BaseGate, Logging, Generic[H], ABC):
-    """ Gate with Hub for connections """
-
-    def __init__(self, delegate: DockerDelegate):
-        super().__init__(delegate=delegate)
-        self.__running = False
-
-    def start(self):
-        self.__running = True
-
-    def stop(self):
-        self.__running = False
-
-    @property
-    def running(self) -> bool:
-        return self.__running
-
-    # # Override
-    # def run(self):
-    #     self.__running = True
-    #     while self.running:
-    #         if not self.process():
-    #             self._idle()
-    #     self.info(msg='gate closing')
-    #
-    # # noinspection PyMethodMayBeStatic
-    # def _idle(self):
-    #     time.sleep(0.25)
-
-    # Override
-    def connection_state_changed(self, previous: ConnectionState, current: ConnectionState, connection: Connection):
-        # debug info
-        if current is None or current == ConnectionState.ERROR:
-            self.error(msg='connection lost: %s -> %s, %s' % (previous, current, connection))
-        elif current != ConnectionState.EXPIRED and current != ConnectionState.MAINTAINING:
-            self.debug(msg='connection state changed: %s -> %s, %s' % (previous, current, connection))
-        super().connection_state_changed(previous=previous, current=current, connection=connection)
-
-    # Override
-    def connection_received(self, data: bytes, connection: Connection):
-        super().connection_received(data=data, connection=connection)
-        self.debug(msg='received %d byte(s): %s' % (len(data), connection))
-
-    # Override
-    def connection_sent(self, sent: int, data: bytes, connection: Connection):
-        super().connection_sent(sent=sent, data=data, connection=connection)
-        self.debug(msg='sent %d byte(s): %s' % (len(data), connection))
-
-    # Override
-    def connection_failed(self, error: Union[IOError, socket.error], data: bytes, connection: Connection):
-        super().connection_failed(error=error, data=data, connection=connection)
-        self.error(msg='failed to send %d byte(s): %s, remote=%s' % (len(data), error, connection.remote_address))
-
-    # Override
-    def connection_error(self, error: Union[IOError, socket.error], connection: Connection):
-        super().connection_error(error=error, connection=connection)
-        if error is not None and str(error).startswith('failed to send: '):
-            self.warning(msg='ignore socket error: %s, remote=%s' % (error, connection.remote_address))
-
-    def get_channel(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]]) -> Optional[Channel]:
-        hub = self.hub
-        assert isinstance(hub, Hub), 'hub error: %s' % hub
-        return hub.open(remote=remote, local=local)
+    def fetch_docker(self, advance_party: List[bytes], remote: SocketAddress, local: Optional[SocketAddress]) -> Docker:
+        # try to get docker
+        with self.__lock:
+            old = self._get_docker(remote=remote, local=local)
+            if old is None:  # and advance_party is not None:
+                # create & cache docker
+                worker = self._create_docker(advance_party, remote=remote, local=local)
+                self._set_docker(worker, remote=remote, local=local)
+            else:
+                worker = old
+        if old is None:
+            hub = self.hub
+            assert isinstance(hub, Hub), 'gate hub error: %s' % hub
+            conn = hub.connect(remote=remote, local=local)
+            if conn is None:
+                # assert False, 'failed to get connection: %s -> %s' % (local, remote)
+                self._remove_docker(worker, remote=remote, local=local)
+                worker = None
+            else:
+                assert isinstance(worker, StarDocker), 'docker error: %s, %s' % (remote, worker)
+                # set connection for this docker
+                worker.set_connection(conn)
+        return worker
 
     def send_response(self, payload: bytes, ship: Arrival,
-                      remote: Tuple[str, int], local: Optional[Tuple[str, int]]) -> bool:
-        worker = self.get_docker(remote=remote, local=local, advance_party=[])
+                      remote: SocketAddress, local: Optional[SocketAddress]) -> bool:
+        worker = self._get_docker(remote=remote, local=local)
         if isinstance(worker, MTPStreamDocker):
             # sn = TransactionID.from_data(data=ship.sn)
             sn = TransactionID.generate()
@@ -193,6 +127,68 @@ class CommonGate(BaseGate, Logging, Generic[H], ABC):
         else:
             raise LookupError('docker error (%s, %s): %s' % (remote, local, worker))
 
+    # Override
+    def _heartbeat(self, connection: Connection):
+        # let the client to do the job
+        if isinstance(connection, ActiveConnection):
+            super()._heartbeat(connection=connection)
+
+    # Override
+    def _cache_advance_party(self, data: bytes, connection: Connection) -> List[bytes]:
+        # TODO: cache the advance party before decide which docker to use
+        if len(data) == 0:
+            return []
+        else:
+            return [data]
+
+    # Override
+    def _clear_advance_party(self, connection: Connection):
+        # TODO: remove advance party for this connection
+        pass
+
+    #
+    #   Connection Delegate
+    #
+
+    # Override
+    def connection_state_changed(self, previous: Optional[ConnectionState], current: Optional[ConnectionState],
+                                 connection: Connection):
+        index = -1 if current is None else current.index
+        if index == StateOrder.ERROR:
+            self.error(msg='connection lost: %s -> %s, %s' % (previous, current, connection.remote_address))
+        elif index != StateOrder.EXPIRED and index != StateOrder.MAINTAINING:
+            self.debug(msg='connection state changed: %s -> %s, %s' % (previous, current, connection.remote_address))
+        try:
+            super().connection_state_changed(previous=previous, current=current, connection=connection)
+        except AssertionError as error:
+            self.error(msg='connection callback failed: %s' % error)
+
+    # Override
+    def connection_received(self, data: bytes, connection: Connection):
+        self.debug(msg='received %d byte(s): %s' % (len(data), connection.remote_address))
+        super().connection_received(data=data, connection=connection)
+
+    # Override
+    def connection_sent(self, sent: int, data: bytes, connection: Connection):
+        super().connection_sent(sent=sent, data=data, connection=connection)
+        self.debug(msg='sent %d byte(s): %s' % (len(data), connection.remote_address))
+
+    # Override
+    def connection_failed(self, error: Union[IOError, socket.error], data: bytes, connection: Connection):
+        super().connection_failed(error=error, data=data, connection=connection)
+        self.error(msg='failed to send %d byte(s): %s, remote=%s' % (len(data), error, connection.remote_address))
+
+    # Override
+    def connection_error(self, error: Union[IOError, socket.error], connection: Connection):
+        super().connection_error(error=error, connection=connection)
+        if error is not None and str(error).startswith('failed to send: '):
+            self.warning(msg='ignore socket error: %s, remote=%s' % (error, connection.remote_address))
+
+    def get_channel(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        hub = self.hub
+        assert isinstance(hub, Hub), 'hub error: %s' % hub
+        return hub.open(remote=remote, local=local)
+
 
 #
 #   Server Gates
@@ -202,23 +198,24 @@ class CommonGate(BaseGate, Logging, Generic[H], ABC):
 class TCPServerGate(CommonGate, Generic[H]):
 
     # Override
-    def _create_docker(self, connection: Connection, advance_party: List[bytes]) -> Optional[Docker]:
-        count = len(advance_party)
+    def _create_docker(self, parties: List[bytes],
+                       remote: SocketAddress, local: Optional[SocketAddress]) -> Docker:
+        count = len(parties)
         if count == 0:
             # return MTPStreamDocker(remote=remote, local=local, gate=self)
-            return None
-        data = advance_party[0]
+            assert False, 'data empty'
+        data = parties[0]
         for i in range(1, count):
-            data = data + advance_party[i]
+            data = data + parties[i]
         if len(data) == 0:
-            return None
+            assert False, 'data empty'
         # check data format before creating docker
         if MTPStreamDocker.check(data=data):
-            docker = MTPStreamDocker(connection=connection)
+            docker = MTPStreamDocker(remote=remote, local=local)
         elif MarsStreamDocker.check(data=data):
-            docker = MarsStreamDocker(connection=connection)
+            docker = MarsStreamDocker(remote=remote, local=local)
         elif WSDocker.check(data=data):
-            docker = WSDocker(connection=connection)
+            docker = WSDocker(remote=remote, local=local)
         else:
             raise LookupError('failed to create docker: %s' % data)
         docker.delegate = self.delegate
@@ -228,15 +225,16 @@ class TCPServerGate(CommonGate, Generic[H]):
 class UDPServerGate(CommonGate, Generic[H]):
 
     # Override
-    def _create_docker(self, connection: Connection, advance_party: List[bytes]) -> Optional[Docker]:
-        count = len(advance_party)
+    def _create_docker(self, parties: List[bytes],
+                       remote: SocketAddress, local: Optional[SocketAddress]) -> Docker:
+        count = len(parties)
         if count == 0:
             # return MTPStreamDocker(remote=remote, local=local, gate=self)
-            return None
-        data = advance_party[count - 1]
+            assert False, 'data empty'
+        data = parties[count - 1]
         # check data format before creating docker
         if MTPStreamDocker.check(data=data):
-            docker = MTPStreamDocker(connection=connection)
+            docker = MTPStreamDocker(remote=remote, local=local)
         else:
             raise LookupError('failed to create docker: %s' % data)
         docker.delegate = self.delegate
@@ -251,8 +249,9 @@ class UDPServerGate(CommonGate, Generic[H]):
 class TCPClientGate(CommonGate, Generic[H]):
 
     # Override
-    def _create_docker(self, connection: Connection, advance_party: List[bytes]) -> Optional[Docker]:
-        docker = MTPStreamDocker(connection=connection)
+    def _create_docker(self, parties: List[bytes],
+                       remote: SocketAddress, local: Optional[SocketAddress]) -> Docker:
+        docker = MTPStreamDocker(remote=remote, local=local)
         docker.delegate = self.delegate
         return docker
 
@@ -260,7 +259,8 @@ class TCPClientGate(CommonGate, Generic[H]):
 class UDPClientGate(CommonGate, Generic[H]):
 
     # Override
-    def _create_docker(self, connection: Connection, advance_party: List[bytes]) -> Optional[Docker]:
-        docker = MTPStreamDocker(connection=connection)
+    def _create_docker(self, parties: List[bytes],
+                       remote: SocketAddress, local: Optional[SocketAddress]) -> Docker:
+        docker = MTPStreamDocker(remote=remote, local=local)
         docker.delegate = self.delegate
         return docker
