@@ -30,12 +30,13 @@
 
 import socket
 import time
+import traceback
 from typing import Optional, Tuple
 
 from dimsdk import ReliableMessage
 
-from startrek.net.channel import get_remote_address, get_local_address
-from startrek import Hub
+from startrek.types import SocketAddress
+from startrek import Channel, Hub
 from startrek import BaseChannel
 from startrek import Connection, ConnectionDelegate, BaseConnection
 from startrek import Arrival, Departure
@@ -44,69 +45,102 @@ from startrek import Docker, DockerStatus, DockerDelegate
 from tcp import StreamChannel
 from tcp import ServerHub, ClientHub
 
+from ..utils import get_remote_address, get_local_address
+from ..utils import get_msg_info
 from ..utils import Runner, Logging
 
 from .protocol import DeparturePacker
 
 from .gate import CommonGate, TCPServerGate, TCPClientGate
-from .queue import MessageQueue
+from .queue import MessageQueue, MessageWrapper
 
 
 class StreamServerHub(ServerHub):
 
     def put_channel(self, channel: StreamChannel):
-        self._set_channel(remote=channel.remote_address, local=channel.local_address, channel=channel)
+        self._set_channel(channel=channel, remote=channel.remote_address, local=None)
 
     # Override
-    def _get_connection(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]]) -> Optional[Connection]:
+    def _get_channel(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        return super()._get_channel(remote=remote, local=None)
+
+    # Override
+    def _set_channel(self, channel: Channel,
+                     remote: Optional[SocketAddress], local: Optional[SocketAddress]):
+        super()._set_channel(channel=channel, remote=remote, local=None)
+
+    # Override
+    def _remove_channel(self, channel: Optional[Channel],
+                        remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        return super()._remove_channel(channel=channel, remote=remote, local=None)
+
+    # Override
+    def _get_connection(self, remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Connection]:
         return super()._get_connection(remote=remote, local=None)
 
     # Override
-    def _set_connection(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]], connection: Connection):
-        super()._set_connection(remote=remote, local=None, connection=connection)
+    def _set_connection(self, connection: Connection,
+                        remote: SocketAddress, local: Optional[SocketAddress]):
+        super()._set_connection(connection=connection, remote=remote, local=None)
 
     # Override
-    def _remove_connection(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]],
-                           connection: Optional[Connection]):
-        super()._remove_connection(remote=remote, local=None, connection=connection)
+    def _remove_connection(self, connection: Optional[Connection],
+                           remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Connection]:
+        return super()._remove_connection(connection=connection, remote=remote, local=None)
 
 
 class StreamClientHub(ClientHub):
 
     def put_channel(self, channel: StreamChannel):
-        self._set_channel(remote=channel.remote_address, local=channel.local_address, channel=channel)
+        self._set_channel(channel=channel, remote=channel.remote_address, local=None)
 
     # Override
-    def _get_connection(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]]) -> Optional[Connection]:
+    def _get_channel(self, remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        return super()._get_channel(remote=remote, local=None)
+
+    # Override
+    def _set_channel(self, channel: Channel,
+                     remote: Optional[SocketAddress], local: Optional[SocketAddress]):
+        super()._set_channel(channel=channel, remote=remote, local=None)
+
+    # Override
+    def _remove_channel(self, channel: Optional[Channel],
+                        remote: Optional[SocketAddress], local: Optional[SocketAddress]) -> Optional[Channel]:
+        return super()._remove_channel(channel=channel, remote=remote, local=None)
+
+    # Override
+    def _get_connection(self, remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Connection]:
         return super()._get_connection(remote=remote, local=None)
 
     # Override
-    def _set_connection(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]], connection: Connection):
-        super()._set_connection(remote=remote, local=None, connection=connection)
+    def _set_connection(self, connection: Connection,
+                        remote: SocketAddress, local: Optional[SocketAddress]):
+        super()._set_connection(connection=connection, remote=remote, local=None)
 
     # Override
-    def _remove_connection(self, remote: Tuple[str, int], local: Optional[Tuple[str, int]],
-                           connection: Optional[Connection]):
-        super()._remove_connection(remote=remote, local=None, connection=connection)
+    def _remove_connection(self, connection: Optional[Connection],
+                           remote: SocketAddress, local: Optional[SocketAddress]) -> Optional[Connection]:
+        return super()._remove_connection(connection=connection, remote=remote, local=None)
 
 
-def reset_send_buffer_size(conn: Connection) -> bool:
-    if not isinstance(conn, BaseConnection):
-        print('[SOCKET] connection error: %s' % conn)
-        return False
-    channel = conn.channel
-    if not isinstance(channel, BaseChannel):
-        print('[SOCKET] channel error: %s, %s' % (channel, conn))
-        return False
-    sock = channel.sock
+def reset_send_buffer_size(conn: Connection = None, sock: socket.socket = None) -> bool:
     if sock is None:
-        print('[SOCKET] socket error: %s, %s' % (sock, conn))
-        return False
+        if not isinstance(conn, BaseConnection):
+            print('[SOCKET] connection error: %s' % conn)
+            return False
+        channel = conn.channel
+        if not isinstance(channel, BaseChannel):
+            print('[SOCKET] channel error: %s, %s' % (channel, conn))
+            return False
+        sock = channel.sock
+        if sock is None:
+            print('[SOCKET] socket error: %s, %s' % (sock, conn))
+            return False
     size = sock.getsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF)
     max_size = GateKeeper.SEND_BUFFER_SIZE
     if size < max_size:
-        print('[SOCKET] change send buffer size: %d -> %d, %s' % (size, max_size, conn))
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, max_size)
+        print('[SOCKET] send buffer size changed: %d -> %d, %s' % (size, max_size, conn))
         return True
     else:
         print('[SOCKET] send buffer size: %d, %s' % (size, conn))
@@ -118,12 +152,12 @@ class GateKeeper(Runner, DockerDelegate, Logging):
     SEND_BUFFER_SIZE = 64 * 1024  # 64 KB
 
     def __init__(self, remote: Tuple[str, int], sock: Optional[socket.socket]):
-        super().__init__()
+        super().__init__(interval=Runner.INTERVAL_SLOW)
         self.__remote = remote
-        self.__gate = self._create_gate(remote=remote, sock=sock)
         self.__queue = MessageQueue()
         self.__active = False
         self.__last_active = 0  # last update time
+        self.__gate = self._create_gate(remote=remote, sock=sock)
 
     def _create_gate(self, remote: Tuple[str, int], sock: Optional[socket.socket]) -> CommonGate:
         if sock is None:
@@ -148,7 +182,8 @@ class GateKeeper(Runner, DockerDelegate, Logging):
             # sock.settimeout(0.5)
             if address is None:
                 address = get_remote_address(sock=sock)
-            channel = StreamChannel(sock=sock, remote=address, local=get_local_address(sock=sock))
+            channel = StreamChannel(remote=address, local=get_local_address(sock=sock))
+            channel.set_socket(sock=sock)
             hub = StreamServerHub(delegate=delegate)
             hub.put_channel(channel=channel)
         return hub
@@ -175,25 +210,25 @@ class GateKeeper(Runner, DockerDelegate, Logging):
             self.__last_active = when
             return True
 
-    @property  # Override
-    def running(self) -> bool:
-        if super().running:
-            return self.gate.running
-
-    # Override
-    def stop(self):
-        super().stop()
-        self.gate.stop()
-
-    # Override
-    def setup(self):
-        super().setup()
-        self.gate.start()
-
-    # Override
-    def finish(self):
-        self.gate.stop()
-        super().finish()
+    # @property  # Override
+    # def running(self) -> bool:
+    #     if super().running:
+    #         return self.gate.running
+    #
+    # # Override
+    # def stop(self):
+    #     super().stop()
+    #     self.gate.stop()
+    #
+    # # Override
+    # def setup(self):
+    #     super().setup()
+    #     self.gate.start()
+    #
+    # # Override
+    # def finish(self):
+    #     self.gate.stop()
+    #     super().finish()
 
     # Override
     def process(self) -> bool:
@@ -209,6 +244,7 @@ class GateKeeper(Runner, DockerDelegate, Logging):
                 return True
         except Exception as e:
             self.error(msg='process error: %s' % e)
+            traceback.print_exc()
             return False
         if not self.active:
             # inactive, wait a while to check again
@@ -230,10 +266,10 @@ class GateKeeper(Runner, DockerDelegate, Logging):
         ok = gate.send_ship(ship=wrapper, remote=self.remote_address, local=None)
         if not ok:
             self.error(msg='gate error, failed to send data')
-        return True
+        return ok
 
     def _docker_pack(self, payload: bytes, priority: int = 0) -> Departure:
-        docker = self.gate.get_docker(remote=self.remote_address, local=None, advance_party=[])
+        docker = self.gate.fetch_docker([], remote=self.remote_address, local=None)
         assert isinstance(docker, DeparturePacker), 'departure packer error: %s' % docker
         return docker.pack(payload=payload, priority=priority)
 
@@ -259,8 +295,12 @@ class GateKeeper(Runner, DockerDelegate, Logging):
 
     # Override
     def docker_failed(self, error: IOError, ship: Departure, docker: Docker):
-        self.error(msg='docker failed to send ship: %s, %s' % (ship, docker))
+        self.error(msg='docker failed to send ship: %s, %s' % (error, docker))
 
     # Override
     def docker_error(self, error: IOError, ship: Departure, docker: Docker):
-        self.error(msg='docker error while sending ship: %s, %s' % (ship, docker))
+        self.error(msg='docker error while sending ship: %s, %s' % (error, docker))
+        if isinstance(ship, MessageWrapper):
+            msg = ship.msg
+            if msg is not None:
+                self.error(msg='error message: %s' % get_msg_info(msg=msg))
