@@ -32,7 +32,7 @@ import threading
 from typing import Optional, Union, List, Tuple
 
 from startrek import Arrival, Departure
-from startrek import Connection
+from startrek.types import SocketAddress
 
 from udp.ba import ByteArray, Data
 from udp.mtp import DataType, TransactionID, Header, Package
@@ -106,8 +106,8 @@ class MTPStreamDeparture(PackageDeparture):
 class MTPStreamDocker(PackageDocker, DeparturePacker):
     """ Docker for MTP packages """
 
-    def __init__(self, connection: Connection):
-        super().__init__(connection=connection)
+    def __init__(self, remote: SocketAddress, local: Optional[SocketAddress]):
+        super().__init__(remote=remote, local=local)
         self.__chunks = Data.ZERO
         self.__chunks_lock = threading.RLock()
         self.__package_received = False
@@ -121,6 +121,7 @@ class MTPStreamDocker(PackageDocker, DeparturePacker):
             # try to fetch a package
             pack, offset = MTPHelper.seek_package(data=data)
             self.__package_received = pack is not None
+            remain_len = len(data)
             if offset >= 0:
                 # 'error part' + 'MTP package' + 'remaining data
                 if pack is not None:
@@ -130,6 +131,7 @@ class MTPStreamDocker(PackageDocker, DeparturePacker):
                 elif offset < data.size:
                     data = data.slice(start=offset)
                     self.__chunks = data.concat(self.__chunks)
+                remain_len -= offset
             return pack
 
     # Override
