@@ -38,6 +38,7 @@
 import threading
 from typing import Optional, List, Dict
 
+from dimsdk import DateTime
 from dimsdk import ReliableMessage
 
 from startrek import Arrival, Departure
@@ -95,6 +96,8 @@ class MessageQueue:
         self.__priorities: List[int] = []
         self.__fleets: Dict[int, List[MessageWrapper]] = {}  # priority => List[MessageWrapper]
         self.__lock = threading.Lock()
+        # purge
+        self.__next_purge_time = 0
 
     def append(self, msg: ReliableMessage, ship: Departure) -> bool:
         priority = ship.priority
@@ -112,7 +115,7 @@ class MessageQueue:
                 signature = msg.get('signature')
                 for wrapper in fleet:
                     item = wrapper.msg
-                    if item is not None and item.get('signature') == signature:
+                    if self.__is_duplicated(item, msg):
                         print('[QUEUE] duplicated message: %s' % signature)
                         return False
             # 2. append with wrapper
@@ -120,6 +123,24 @@ class MessageQueue:
             fleet.append(wrapper)
             return True
 
+    # noinspection PyMethodMayBeStatic
+    def __is_duplicated(self, msg1: ReliableMessage, msg2: ReliableMessage) -> bool:
+        if msg1 is None or msg2 is None:
+            return False
+        sig1 = msg1.get('signature')
+        sig2 = msg2.get('signature')
+        if sig1 is None or sig2 is None:
+            # assert False, 'signature should not empty here: %s, %s' % (msg1, msg2)
+            return False
+        elif sig1 != sig2:
+            return False
+        # maybe it's a group message split for every members,
+        # so we still need to check receiver here.
+        to1 = msg1.receiver
+        to2 = msg2.receiver
+        assert to1 is not None and to2 is not None, 'receiver should not empty here: %s, %s' % (msg1, msg2)
+        return to1 == to2
+
     def __insert(self, priority: int):
         index = 0
         for value in self.__priorities:
@@ -147,6 +168,12 @@ class MessageQueue:
                     return fleet.pop(0)
 
     def purge(self):
+        now = DateTime.now()
+        if now < self.__next_purge_time:
+            return -1
+        else:
+            # next purge after half a minute
+            self.__next_purge_time = now.timestamp + 30
         with self.__lock:
             priorities = list(self.__priorities)
             for prior in priorities:
