@@ -33,17 +33,13 @@ import weakref
 from abc import ABC
 from typing import Optional, Tuple
 
-from dimsdk import EntityType, ID, Content
+from dimsdk import ID, Content
 from dimsdk import InstantMessage, ReliableMessage
 
-from startrek import Docker, Departure
-
 from ..common import CommonMessenger
 from ..common import Session, SessionDBI
-from ..common import MessageDBI
 
 from .gatekeeper import GateKeeper
-from .queue import MessageWrapper
 
 
 class BaseSession(GateKeeper, Session, ABC):
@@ -69,14 +65,14 @@ class BaseSession(GateKeeper, Session, ABC):
             return True
 
     @property
-    def messenger(self) -> CommonMessenger:
+    def messenger(self) -> Optional[CommonMessenger]:
         ref = self.__messenger
-        assert ref is not None, 'messenger not set yet'
-        return ref()
+        if ref is not None:
+            return ref()
 
     @messenger.setter
     def messenger(self, transceiver: CommonMessenger):
-        self.__messenger = weakref.ref(transceiver)
+        self.__messenger = None if transceiver is None else weakref.ref(transceiver)
 
     # Override
     def queue_message_package(self, msg: ReliableMessage, data: bytes, priority: int = 0) -> bool:
@@ -102,47 +98,3 @@ class BaseSession(GateKeeper, Session, ABC):
     def send_reliable_message(self, msg: ReliableMessage, priority: int = 0) -> bool:
         messenger = self.messenger
         return messenger.send_reliable_message(msg=msg, priority=priority)
-
-    #
-    #   Docker Delegate
-    #
-
-    # Override
-    def docker_sent(self, ship: Departure, docker: Docker):
-        if isinstance(ship, MessageWrapper):
-            msg = ship.msg
-            if msg is not None:
-                # remove from database for actual receiver
-                receiver = self.identifier
-                db = self.messenger.database
-                remove_reliable_message(msg=msg, receiver=receiver, database=db)
-
-
-def remove_reliable_message(msg: ReliableMessage, receiver: ID, database: MessageDBI):
-    # 0. if session ID is empty, means user not login;
-    #    this message must be a handshake command, and
-    #    its receiver must be the targeted user.
-    # 1. if this session is a station, check original receiver;
-    #    a message to station won't be stored.
-    # 2. if the msg.receiver is a different user ID, means it's
-    #    a roaming message, remove it for actual receiver.
-    # 3. if the original receiver is a group, it must had been
-    #    replaced to the group assistant ID by GroupDeliver.
-    if receiver is None or receiver.type == EntityType.STATION:
-        # if msg.receiver == receiver:
-        #     # station message won't be stored
-        #     return False
-        receiver = msg.receiver
-    sig = get_sig(msg=msg)
-    print('[QUEUE] message (%s) sent, remove from db: %s => %s (%s)'
-          % (sig, msg.sender, msg.receiver, receiver))
-    # remove sent message from database
-    return database.remove_reliable_message(msg=msg, receiver=receiver)
-
-
-def get_sig(msg: ReliableMessage) -> str:
-    """ last 6 bytes (signature in base64) """
-    sig = msg.get('signature')
-    # assert isinstance(sig, str), 'signature error: %s' % sig
-    sig = sig.strip()
-    return sig[-8:]  # last 6 bytes (signature in base64)
