@@ -38,12 +38,15 @@ from .t_meta import MetaTable
 from .t_document import DocumentTable
 from .t_user import UserTable
 from .t_group import GroupTable
+from .t_group_history import GroupHistoryTable
+
+from .t_group_keys import GroupKeysTable
 
 from .t_message import ReliableMessageTable
 from .t_cipherkey import CipherKeyTable
 
 from .t_login import LoginTable
-from .t_provider import ProviderTable
+from .t_station import StationTable
 
 from .account import AccountDatabase
 from .message import MessageDatabase
@@ -55,30 +58,34 @@ __all__ = [
     #   DBI
     #
     'PrivateKeyDBI', 'MetaDBI', 'DocumentDBI',
-    'UserDBI', 'GroupDBI',
+    'UserDBI', 'ContactDBI', 'GroupDBI', 'GroupHistoryDBI',
     'AccountDBI',
 
-    'ReliableMessageDBI', 'CipherKeyDBI',
+    'ReliableMessageDBI', 'CipherKeyDBI', 'GroupKeysDBI',
     'MessageDBI',
 
-    'LoginDBI', 'ProviderDBI',
+    'LoginDBI', 'ProviderDBI', 'SessionDBI',
     'SessionDBI',
+    'ProviderInfo', 'StationInfo',
 
     #
     #   DOS
     #
     'Storage',
     'PrivateKeyStorage', 'MetaStorage', 'DocumentStorage',
-    'UserStorage', 'GroupStorage',
+    'UserStorage', 'GroupStorage', 'GroupHistoryStorage',
+    'GroupKeysStorage',
     'LoginStorage',
+    'StationStorage',
 
     #
     #   Table
     #
     'PrivateKeyTable', 'MetaTable', 'DocumentTable',
-    'UserTable', 'GroupTable',
+    'UserTable', 'GroupTable', 'GroupHistoryTable',
+    'GroupKeysTable',
     'ReliableMessageTable', 'CipherKeyTable',
-    'LoginTable', 'ProviderTable',
+    'LoginTable', 'StationTable',
 
     #
     #   Database
