@@ -28,10 +28,10 @@ from typing import Optional, Tuple
 from dimsdk import ID
 from dimsdk import ReliableMessage
 
+from ...utils import template_replace
 from ...common import LoginDBI, LoginCommand
 
 from .base import Storage
-from .base import template_replace
 
 
 class LoginStorage(Storage, LoginDBI):
@@ -44,26 +44,25 @@ class LoginStorage(Storage, LoginDBI):
     login_path = '{PUBLIC}/{ADDRESS}/login.js'
 
     def show_info(self):
-        path = template_replace(self.login_path, 'PUBLIC', self._public)
-        print('!!! login cmd path: %s' % path)
+        path = self.public_path(self.login_path)
+        print('!!!      login cmd path: %s' % path)
 
     def __login_path(self, identifier: ID) -> str:
-        path = self.login_path
-        path = template_replace(path, 'PUBLIC', self._public)
-        return template_replace(path, 'ADDRESS', str(identifier.address))
+        path = self.public_path(self.login_path)
+        return template_replace(path, key='ADDRESS', value=str(identifier.address))
 
     #
     #   Login DBI
     #
 
     # Override
-    def login_command_message(self, identifier: ID) -> Tuple[Optional[LoginCommand], Optional[ReliableMessage]]:
+    def login_command_message(self, user: ID) -> Tuple[Optional[LoginCommand], Optional[ReliableMessage]]:
         """ load login command from file """
-        path = self.__login_path(identifier=identifier)
+        path = self.__login_path(identifier=user)
         self.info(msg='Loading login command from: %s' % path)
         info = self.read_json(path=path)
         if info is None:
-            # login command not found
+            # command not found
             return None, None
         cmd = info.get('cmd')
         msg = info.get('msg')
@@ -72,12 +71,12 @@ class LoginStorage(Storage, LoginDBI):
         return cmd, ReliableMessage.parse(msg=msg)
 
     # Override
-    def save_login_command_message(self, identifier: ID, content: LoginCommand, msg: ReliableMessage) -> bool:
+    def save_login_command_message(self, user: ID, content: LoginCommand, msg: ReliableMessage) -> bool:
         """ save login command into file """
         info = {
             'cmd': content.dictionary,
             'msg': msg.dictionary
         }
-        path = self.__login_path(identifier=identifier)
+        path = self.__login_path(identifier=user)
         self.info(msg='Saving login command into: %s' % path)
         return self.write_json(container=info, path=path)
