@@ -28,10 +28,10 @@ from typing import Optional, List
 from dimsdk import PrivateKey, SignKey, DecryptKey
 from dimsdk import ID
 
+from ...utils import template_replace
 from ...common import PrivateKeyDBI
 
 from .base import Storage
-from .base import template_replace
 
 
 class PrivateKeyStorage(Storage, PrivateKeyDBI):
@@ -54,20 +54,18 @@ class PrivateKeyStorage(Storage, PrivateKeyDBI):
     MSG_KEY_TAG = 'V'  # private key pared to visa.key
 
     def show_info(self):
-        path1 = template_replace(self.id_key_path, 'PRIVATE', self._private)
-        path2 = template_replace(self.msg_keys_path, 'PRIVATE', self._private)
-        print('!!!    id key path: %s' % path1)
-        print('!!!  msg keys path: %s' % path2)
+        path1 = self.private_path(self.id_key_path)
+        path2 = self.private_path(self.msg_keys_path)
+        print('!!!         id key path: %s' % path1)
+        print('!!!       msg keys path: %s' % path2)
 
     def __id_key_path(self, identifier: ID) -> str:
-        path = self.id_key_path
-        path = template_replace(path, 'PRIVATE', self._private)
-        return template_replace(path, 'ADDRESS', str(identifier.address))
+        path = self.private_path(self.id_key_path)
+        return template_replace(path, key='ADDRESS', value=str(identifier.address))
 
     def __msg_keys_path(self, identifier: ID) -> str:
-        path = self.msg_keys_path
-        path = template_replace(path, 'PRIVATE', self._private)
-        return template_replace(path, 'ADDRESS', str(identifier.address))
+        path = self.private_path(self.msg_keys_path)
+        return template_replace(path, key='ADDRESS', value=str(identifier.address))
 
     def _save_id_key(self, key: PrivateKey, identifier: ID) -> bool:
         path = self.__id_key_path(identifier=identifier)
@@ -83,7 +81,7 @@ class PrivateKeyStorage(Storage, PrivateKeyDBI):
 
     def _save_msg_key(self, key: PrivateKey, identifier: ID) -> bool:
         private_keys = self._load_msg_keys(identifier=identifier)
-        private_keys = insert_key(key=key, private_keys=private_keys)
+        private_keys = PrivateKeyDBI.insert(item=key, array=private_keys)
         if private_keys is None:
             # nothing changed
             return False
@@ -109,53 +107,29 @@ class PrivateKeyStorage(Storage, PrivateKeyDBI):
     #
 
     # Override
-    def save_private_key(self, key: PrivateKey, identifier: ID, key_type: str = 'M') -> bool:
+    def save_private_key(self, key: PrivateKey, user: ID, key_type: str = 'M') -> bool:
         if key_type == self.ID_KEY_TAG:
             # save private key for meta
-            return self._save_id_key(key=key, identifier=identifier)
+            return self._save_id_key(key=key, identifier=user)
         else:
             # save private key for visa
-            return self._save_msg_key(key=key, identifier=identifier)
+            return self._save_msg_key(key=key, identifier=user)
 
     # Override
-    def private_keys_for_decryption(self, identifier: ID) -> List[DecryptKey]:
-        keys: list = self._load_msg_keys(identifier=identifier)
+    def private_keys_for_decryption(self, user: ID) -> List[DecryptKey]:
+        keys: list = self._load_msg_keys(identifier=user)
         # the 'ID key' could be used for encrypting message too (RSA),
         # so we append it to the decrypt keys here
-        id_key = self._load_id_key(identifier=identifier)
-        if isinstance(id_key, DecryptKey) and find_key(key=id_key, private_keys=keys) < 0:
+        id_key = self._load_id_key(identifier=user)
+        if isinstance(id_key, DecryptKey) and PrivateKeyDBI.find(item=id_key, array=keys) < 0:
             keys.append(id_key)
         return keys
 
     # Override
-    def private_key_for_signature(self, identifier: ID) -> Optional[SignKey]:
+    def private_key_for_signature(self, user: ID) -> Optional[SignKey]:
         # TODO: support multi private keys
-        return self.private_key_for_visa_signature(identifier=identifier)
+        return self.private_key_for_visa_signature(user=user)
 
     # Override
-    def private_key_for_visa_signature(self, identifier: ID) -> Optional[SignKey]:
-        return self._load_id_key(identifier=identifier)
-
-
-#   insert_key(key: PrivateKey, private_keys: List[PrivateKey]) -> Optional[List[PrivateKey]]:
-def insert_key(key: PrivateKey, private_keys: list) -> Optional[List[PrivateKey]]:
-    index = find_key(key=key, private_keys=private_keys)
-    if index == 0:
-        return None  # nothing changed
-    elif index > 0:
-        private_keys.pop(index)  # move to the front
-    elif len(private_keys) > 2:
-        private_keys.pop()  # keep only last three records
-    private_keys.insert(0, key)
-    return private_keys
-
-
-def find_key(key, private_keys: List[PrivateKey]) -> int:
-    index = 0
-    data = key.get('data')
-    for item in private_keys:
-        if item.get('data') == data:
-            return index
-        else:
-            index += 1
-    return -1
+    def private_key_for_visa_signature(self, user: ID) -> Optional[SignKey]:
+        return self._load_id_key(identifier=user)
