@@ -27,37 +27,28 @@ from typing import List
 
 from dimsdk import ID
 
-from ...common import UserDBI
+from ...utils import template_replace
+from ...common import UserDBI, ContactDBI
 
 from .base import Storage
-from .base import template_replace
 
 
-class UserStorage(Storage, UserDBI):
+class UserStorage(Storage, UserDBI, ContactDBI):
     """
         User Storage
         ~~~~~~~~~~~~
-        file path: '.dim/private/users.js'
         file path: '.dim/private/{ADDRESS}/contacts.js'
     """
 
-    users_path = '{PRIVATE}/users.js'
     contacts_path = '{PRIVATE}/{ADDRESS}/contacts.js'
 
     def show_info(self):
-        path1 = template_replace(self.users_path, 'PRIVATE', self._private)
-        path2 = template_replace(self.contacts_path, 'PRIVATE', self._private)
-        print('!!!     users path: %s' % path1)
-        print('!!!  contacts path: %s' % path2)
-
-    def __users_path(self) -> str:
-        path = self.users_path
-        return template_replace(path, 'PRIVATE', self._private)
+        path = self.private_path(self.contacts_path)
+        print('!!!       contacts path: %s' % path)
 
     def __contacts_path(self, identifier: ID) -> str:
-        path = self.contacts_path
-        path = template_replace(path, 'PRIVATE', self._private)
-        return template_replace(path, 'ADDRESS', str(identifier.address))
+        path = self.private_path(self.contacts_path)
+        return template_replace(path, key='ADDRESS', value=str(identifier.address))
 
     #
     #   User DBI
@@ -65,26 +56,20 @@ class UserStorage(Storage, UserDBI):
 
     # Override
     def local_users(self) -> List[ID]:
-        """ load users from file """
-        path = self.__users_path()
-        self.info(msg='Loading users from: %s' % path)
-        users = self.read_json(path=path)
-        if users is None:
-            # local users not found
-            return []
-        return ID.convert(array=users)
+        return []
 
     # Override
     def save_local_users(self, users: List[ID]) -> bool:
-        """ save local users into file """
-        path = self.__users_path()
-        self.info(msg='Saving local users into: %s' % path)
-        return self.write_json(container=ID.revert(array=users), path=path)
+        pass
+
+    #
+    #   Contact DBI
+    #
 
     # Override
-    def contacts(self, identifier: ID) -> List[ID]:
+    def contacts(self, user: ID) -> List[ID]:
         """ load contacts from file """
-        path = self.__contacts_path(identifier=identifier)
+        path = self.__contacts_path(identifier=user)
         self.info(msg='Loading contacts from: %s' % path)
         contacts = self.read_json(path=path)
         if contacts is None:
@@ -93,8 +78,8 @@ class UserStorage(Storage, UserDBI):
         return ID.convert(array=contacts)
 
     # Override
-    def save_contacts(self, contacts: List[ID], identifier: ID) -> bool:
+    def save_contacts(self, contacts: List[ID], user: ID) -> bool:
         """ save contacts into file """
-        path = self.__contacts_path(identifier=identifier)
+        path = self.__contacts_path(identifier=user)
         self.info(msg='Saving contacts into: %s' % path)
         return self.write_json(container=ID.revert(array=contacts), path=path)
