@@ -23,13 +23,14 @@
 # SOFTWARE.
 # ==============================================================================
 
-import time
 from typing import Optional, Tuple
 
+from dimsdk import DateTime
 from dimsdk import ID
 from dimsdk import ReliableMessage
 
 from ..utils import CacheManager
+from ..utils import is_before
 from ..common import LoginDBI, LoginCommand
 
 from .dos import LoginStorage
@@ -38,48 +39,65 @@ from .dos import LoginStorage
 class LoginTable(LoginDBI):
     """ Implementations of LoginDBI """
 
+    CACHE_EXPIRES = 300    # seconds
+    CACHE_REFRESHING = 32  # seconds
+
     def __init__(self, root: str = None, public: str = None, private: str = None):
         super().__init__()
+        self.__dos = LoginStorage(root=root, public=public, private=private)
         man = CacheManager()
         self.__login_cache = man.get_pool(name='login')  # ID => (LoginCommand, ReliableMessage)
-        self.__login_storage = LoginStorage(root=root, public=public, private=private)
 
     def show_info(self):
-        self.__login_storage.show_info()
+        self.__dos.show_info()
+
+    def _is_expired(self, user: ID, content: LoginCommand) -> bool:
+        """ check old record with command time """
+        new_time = content.time
+        if new_time is None or new_time <= 0:
+            return False
+        # check old record
+        old, _ = self.login_command_message(user=user)
+        if old is not None and is_before(old_time=old.time, new_time=new_time):
+            # command expired
+            return True
+
+    #
+    #   Login DBI
+    #
 
     # Override
-    def login_command_message(self, identifier: ID) -> Tuple[Optional[LoginCommand], Optional[ReliableMessage]]:
+    def save_login_command_message(self, user: ID, content: LoginCommand, msg: ReliableMessage) -> bool:
+        # 0. check command time
+        if self._is_expired(user=user, content=content):
+            # command expired, drop it
+            return False
+        # 1. store into memory cache
+        self.__login_cache.update(key=user, value=(content, msg), life_span=self.CACHE_EXPIRES)
+        # 2. store into local storage
+        return self.__dos.save_login_command_message(user=user, content=content, msg=msg)
+
+    # Override
+    def login_command_message(self, user: ID) -> Tuple[Optional[LoginCommand], Optional[ReliableMessage]]:
         """ get login command message for user """
-        now = time.time()
+        now = DateTime.now()
         # 1. check memory cache
-        value, holder = self.__login_cache.fetch(key=identifier, now=now)
+        value, holder = self.__login_cache.fetch(key=user, now=now)
         if value is None:
             # cache empty
             if holder is None:
-                # login command not load yet, wait to load
-                self.__login_cache.update(key=identifier, life_span=128, now=now)
+                # cache not load yet, wait to load
+                self.__login_cache.update(key=user, life_span=self.CACHE_REFRESHING, now=now)
             else:
                 if holder.is_alive(now=now):
-                    # login command not exists
+                    # cache not exists
                     return None, None
-                # login command expired, wait to reload
-                holder.renewal(duration=128, now=now)
+                # cache expired, wait to reload
+                holder.renewal(duration=self.CACHE_REFRESHING, now=now)
             # 2. check local storage
-            cmd, msg = self.__login_storage.login_command_message(identifier=identifier)
+            cmd, msg = self.__dos.login_command_message(user=user)
             value = (cmd, msg)
             # 3. update memory cache
-            self.__login_cache.update(key=identifier, value=value, life_span=36000, now=now)
+            self.__login_cache.update(key=user, value=value, life_span=self.CACHE_EXPIRES, now=now)
         # OK, return cached value
         return value
-
-    # Override
-    def save_login_command_message(self, identifier: ID, content: LoginCommand, msg: ReliableMessage) -> bool:
-        # 1. check old record
-        old, _ = self.login_command_message(identifier=identifier)
-        if isinstance(old, LoginCommand) and old.time >= content.time > 0:
-            # command expired
-            return False
-        # 2. store into memory cache
-        self.__login_cache.update(key=identifier, value=(content, msg), life_span=36000)
-        # 3. store into local storage
-        return self.__login_storage.save_login_command_message(identifier=identifier, content=content, msg=msg)
