@@ -23,9 +23,9 @@
 # SOFTWARE.
 # ==============================================================================
 
-import time
-from typing import List, Tuple
+from typing import List
 
+from dimsdk import DateTime
 from dimsdk import ID
 from dimsdk import ReliableMessage
 
@@ -36,63 +36,59 @@ from ..common import ReliableMessageDBI
 class ReliableMessageTable(ReliableMessageDBI):
     """ Implementations of ReliableMessageDBI """
 
-    CACHE_LIMIT = 71680  # only cache last messages
+    CACHE_EXPIRES = 3600*24*7  # seconds
 
     # noinspection PyUnusedLocal
     def __init__(self, root: str = None, public: str = None, private: str = None):
         super().__init__()
         man = CacheManager()
-        self.__msg_cache = man.get_pool(name='message')  # ID => List[ReliableMessage]
+        self.__msg_cache = man.get_pool(name='reliable_messages')  # ID => List[ReliableMessage]
 
     # noinspection PyMethodMayBeStatic
     def show_info(self):
         print('!!! messages cached in memory only !!!')
 
     #
-    #   ReliableMessage DBI
+    #   Reliable Message DBI
     #
 
     # Override
-    def reliable_messages(self, receiver: ID, start: int = 0, limit: int = 1024) -> Tuple[List[ReliableMessage], int]:
-        now = time.time()
+    def reliable_messages(self, receiver: ID, limit: int = 1024) -> List[ReliableMessage]:
+        now = DateTime.now()
         # get all messages
         messages, _ = self.__msg_cache.fetch(key=receiver, now=now)
         if messages is None:
-            messages = []
-            total = 0
-        else:
-            total = len(messages)
-        # check range: [start, end)
-        start, end = get_range(start=start, limit=limit, total=total)
-        if 0 < start or end < total:
-            messages = messages[start:end]
-        # return message list and remaining count
-        return messages, total - end
+            return []
+        # only last cached messages will be returned
+        if 0 < limit < len(messages):
+            messages = messages[-limit:]
+        return messages
 
     # Override
     def cache_reliable_message(self, msg: ReliableMessage, receiver: ID) -> bool:
-        now = time.time()
+        now = DateTime.now()
         # assert receiver.is_user, 'message receiver error: %s' % receiver
         messages, holder = self.__msg_cache.fetch(key=receiver, now=now)
         if messages is None:
             messages = [msg]
-            self.__msg_cache.update(key=receiver, value=messages, life_span=3600*24*7, now=now)
-        elif find_message(msg=msg, messages=messages) >= 0:
-            # duplicated
-            return False
-        else:
-            assert isinstance(messages, list), 'msg cache list error: %s' % messages
-            while len(messages) > self.CACHE_LIMIT:
+            self.__msg_cache.update(key=receiver, value=messages, life_span=self.CACHE_EXPIRES, now=now)
+            return True
+        elif find_message(msg=msg, messages=messages) < 0:
+            assert isinstance(messages, List), 'msg cache list error: %s' % messages
+            while len(messages) > ReliableMessageDBI.CACHE_LIMIT:
                 # overflow
                 messages.pop(0)
             # append to tail
             messages.append(msg)
             holder.update(value=messages, now=now)
-        return True
+            return True
+        else:
+            # duplicated
+            return False
 
     # Override
     def remove_reliable_message(self, msg: ReliableMessage, receiver: ID) -> bool:
-        now = time.time()
+        now = DateTime.now()
         # assert receiver.is_user, 'message receiver error: %s' % receiver
         messages, holder = self.__msg_cache.fetch(key=receiver, now=now)
         if messages is None:
@@ -118,25 +114,3 @@ def find_message(msg: ReliableMessage, messages: List[ReliableMessage]) -> int:
         index += 1
     # not found
     return -1
-
-
-def get_range(start: int, limit: int, total: int) -> Tuple[int, int]:
-    """ make range: [start, end) """
-    assert limit > 0, 'limit error: %d' % limit
-    assert total >= 0, 'total count error: %d' % total
-    # seek for start
-    if start < 0:
-        # seek from the tail
-        start += total
-        if start < 0:
-            # too few data, start from the head
-            start = 0
-    elif start > total:
-        # out of range
-        start = total
-    # seek for end
-    end = start + limit
-    if end > total:
-        # too few data, stop at the tail
-        end = total
-    return start, end
