@@ -23,9 +23,9 @@
 # SOFTWARE.
 # ==============================================================================
 
-import time
 from typing import List
 
+from dimsdk import DateTime
 from dimsdk import ID
 
 from ..utils import CacheManager
@@ -37,15 +37,17 @@ from .dos import UserStorage
 class UserTable(UserDBI):
     """ Implementations of UserDBI """
 
+    CACHE_EXPIRES = 300    # seconds
+    CACHE_REFRESHING = 32  # seconds
+
     def __init__(self, root: str = None, public: str = None, private: str = None):
         super().__init__()
+        self.__dos = UserStorage(root=root, public=public, private=private)
         man = CacheManager()
-        self.__dim_cache = man.get_pool(name='dim')            # 'local_users' => List[ID]
         self.__contacts_cache = man.get_pool(name='contacts')  # ID => List[ID]
-        self.__user_storage = UserStorage(root=root, public=public, private=private)
 
     def show_info(self):
-        self.__user_storage.show_info()
+        self.__dos.show_info()
 
     #
     #   User DBI
@@ -53,62 +55,43 @@ class UserTable(UserDBI):
 
     # Override
     def local_users(self) -> List[ID]:
-        """ get local users """
-        now = time.time()
-        # 1. check memory cache
-        value, holder = self.__dim_cache.fetch(key='local_users', now=now)
-        if value is None:
-            # cache empty
-            if holder is None:
-                # local users not load yet, wait to load
-                self.__dim_cache.update(key='local_users', life_span=128, now=now)
-            else:
-                if holder.is_alive(now=now):
-                    # local users not exists
-                    return []
-                # local users expired, wait to reload
-                holder.renewal(duration=128, now=now)
-            # 2. check local storage
-            value = self.__user_storage.local_users()
-            # 3. update memory cache
-            self.__dim_cache.update(key='local_users', value=value, life_span=36000, now=now)
-        # OK, return cached value
-        return value
+        return []
 
     # Override
     def save_local_users(self, users: List[ID]) -> bool:
-        # 1. store into memory cache
-        self.__dim_cache.update(key='local_users', value=users, life_span=36000)
-        # 2. store into local storage
-        return self.__user_storage.save_local_users(users=users)
+        pass
+
+    #
+    #   Contact DBI
+    #
 
     # Override
-    def contacts(self, identifier: ID) -> List[ID]:
+    def contacts(self, user: ID) -> List[ID]:
         """ get contacts for user """
-        now = time.time()
+        now = DateTime.now()
         # 1. check memory cache
-        value, holder = self.__contacts_cache.fetch(key=identifier, now=now)
+        value, holder = self.__contacts_cache.fetch(key=user, now=now)
         if value is None:
             # cache empty
             if holder is None:
                 # contacts not load yet, wait to load
-                self.__contacts_cache.update(key=identifier, life_span=128, now=now)
+                self.__contacts_cache.update(key=user, life_span=self.CACHE_REFRESHING, now=now)
             else:
                 if holder.is_alive(now=now):
                     # contacts not exists
                     return []
                 # contacts expired, wait to reload
-                holder.renewal(duration=128, now=now)
+                holder.renewal(duration=self.CACHE_REFRESHING, now=now)
             # 2. check local storage
-            value = self.__user_storage.contacts(identifier=identifier)
+            value = self.__dos.contacts(user=user)
             # 3. update memory cache
-            self.__contacts_cache.update(key=identifier, value=value, life_span=36000, now=now)
+            self.__contacts_cache.update(key=user, value=value, life_span=self.CACHE_EXPIRES, now=now)
         # OK, return cached value
         return value
 
     # Override
-    def save_contacts(self, contacts: List[ID], identifier: ID) -> bool:
+    def save_contacts(self, contacts: List[ID], user: ID) -> bool:
         # 1. store into memory cache
-        self.__contacts_cache.update(key=identifier, value=contacts, life_span=36000)
+        self.__contacts_cache.update(key=user, value=contacts, life_span=self.CACHE_EXPIRES)
         # 2. store into local storage
-        return self.__user_storage.save_contacts(contacts=contacts, identifier=identifier)
+        return self.__dos.save_contacts(contacts=contacts, user=user)
