@@ -31,7 +31,6 @@
 """
 
 import threading
-import time
 import weakref
 from abc import ABC, abstractmethod
 from typing import Optional, List, Set
@@ -39,15 +38,17 @@ from typing import Optional, List, Set
 from dimsdk import ContentType
 from dimsdk import EntityType, ID
 from dimsdk import ReliableMessage
+from dimsdk import Station
 
-from ..utils import Logging
+from ..utils import Log, Logging
 from ..utils import Runner
-from ..common import CommonFacebook
+from ..utils import get_msg_sig
+from ..common import ProviderInfo
 from ..common import MessageDBI, SessionDBI
 from ..common import HandshakeCommand
-from ..conn.session import get_sig
 
 from ..client import ClientSession
+from ..client import ClientFacebook
 from ..client import ClientMessenger
 from ..client import ClientMessagePacker
 from ..client import ClientMessageProcessor
@@ -60,7 +61,7 @@ from .shared import create_session
 class Octopus(Runner, Logging):
 
     def __init__(self, shared: GlobalVariable, local_host: str = '127.0.0.1', local_port: int = 9394):
-        super().__init__()
+        super().__init__(interval=60)
         self.__shared = shared
         self.__inner = self.create_inner_terminal(host=local_host, port=local_port)
         self.__outers: Set[Terminal] = set()
@@ -92,6 +93,9 @@ class Octopus(Runner, Logging):
         messenger = create_messenger(facebook=shared.facebook, database=shared.mdb,
                                      session=session, messenger_class=InnerMessenger)
         messenger.octopus = self
+        # set for archivist
+        archivist = self.shared.facebook.archivist
+        archivist.messenger = messenger
         return create_terminal(messenger=messenger)
 
     def create_outer_terminal(self, host: str, port: int) -> Terminal:
@@ -108,9 +112,21 @@ class Octopus(Runner, Logging):
             self.__outer_map[identifier] = terminal
 
     def connect(self, host: str, port: int = 9394):
-        terminal = self.create_outer_terminal(host=host, port=port)
+        # create a new terminal for remote host:port
         with self.__outer_lock:
+            # check exist terminals
+            outers = self.__outers.copy()
+            for out in outers:
+                # check station
+                station = out.session.station
+                if port == station.port and host == station.host:
+                    self.warning(msg='connection already exists: (%s, %d)' % (host, port))
+                    # self.__outers.discard(out)
+                    return None
+            # create new terminal
+            terminal = self.create_outer_terminal(host=host, port=port)
             self.__outers.add(terminal)
+            return terminal
 
     def start(self):
         thread = threading.Thread(target=self.run)
@@ -127,56 +143,60 @@ class Octopus(Runner, Logging):
         for out in outers:
             out.stop()
 
-    # Override
-    def _idle(self):
-        time.sleep(60)
-
     # Override
     def process(self) -> bool:
         # get all neighbor stations
         db = self.database
-        neighbors = db.all_neighbors()
+        providers = db.all_providers()
+        assert len(providers) > 0, 'service provider not found'
+        gsp = providers[0].identifier
+        neighbors = db.all_stations(provider=gsp)
+        if neighbors is not None:
+            neighbors = neighbors.copy()
         # get all outer terminals
         with self.__outer_lock:
             outers = set(self.__outers)
+        self.debug(msg='checking %d client(s) with %d neighbor(s)' % (len(outers), len(neighbors)))
         for out in outers:
             # check station
             station = out.session.station
+            sid = station.identifier
             host = station.host
             port = station.port
+            # reduce neighbors
             for item in neighbors:
-                if item[0] == host and item[1] == port:
+                if item.port == port and item.host == host:
                     # got
-                    neighbors.discard(item)
+                    neighbors.remove(item)
                     break
-            if out.is_alive:
-                # outer terminal alive, ignore it
+            # check outer client
+            if out.running:
+                # skip running client
                 continue
-            # remove dead terminal
-            sid = station.identifier
+            else:
+                # remove dead client
+                self.warning(msg='client stopped, remove it: %s (%s:%d)' % (sid, host, port))
             with self.__outer_lock:
                 self.__outers.discard(out)
                 if sid is not None:
                     self.__outer_map.pop(sid, None)
         # check new neighbors
         for item in neighbors:
-            host = item[0]
-            port = item[1]
-            sid = item[2]
-            self.info(msg='connecting neighbor station "%s" (%s:%d)' % (sid, host, port))
+            host = item.host
+            port = item.port
+            self.debug(msg='connecting neighbor station (%s:%d), client count: %d' % (host, port, len(self.__outers)))
             self.connect(host=host, port=port)
         return False
 
     def income_message(self, msg: ReliableMessage, priority: int = 0) -> List[ReliableMessage]:
         """ redirect message from remote station """
+        sig = get_msg_sig(msg=msg)
         messenger = self.inner_messenger
         if messenger.send_reliable_message(msg=msg, priority=priority):
-            sig = get_sig(msg=msg)
             self.info(msg='redirected msg (%s) for receiver (%s)' % (sig, msg.receiver))
-            # no need to respond receipt for station
-            return []
-        sig = get_sig(msg=msg)
-        self.error(msg='failed to redirect msg (%s) for receiver (%s)' % (sig, msg.receiver))
+        else:
+            self.error(msg='failed to redirect msg (%s) for receiver (%s)' % (sig, msg.receiver))
+        # no need to respond receipt for station
         return []
 
     def outgo_message(self, msg: ReliableMessage, priority: int = 0) -> List[ReliableMessage]:
@@ -193,11 +213,11 @@ class Octopus(Runner, Logging):
             return []
         msg.pop('neighbor', None)
         if messenger.send_reliable_message(msg=msg, priority=priority):
-            sig = get_sig(msg=msg)
+            sig = get_msg_sig(msg=msg)
             self.info(msg='redirected msg (%s) to target (%s) for receiver (%s)' % (sig, target, msg.receiver))
             # no need to respond receipt for station
             return []
-        sig = get_sig(msg=msg)
+        sig = get_msg_sig(msg=msg)
         self.error(msg='failed to redirect msg (%s) to target (%s) for receiver (%s)' % (sig, target, msg.receiver))
         return []
 
@@ -205,7 +225,7 @@ class Octopus(Runner, Logging):
 class OctopusMessenger(ClientMessenger, ABC):
     """ Messenger for processing message from remote station """
 
-    def __init__(self, session: ClientSession, facebook: CommonFacebook, database: MessageDBI):
+    def __init__(self, session: ClientSession, facebook: ClientFacebook, database: MessageDBI):
         super().__init__(session=session, facebook=facebook, database=database)
         self.__terminal: Optional[weakref.ReferenceType] = None
         self.__octopus: Optional[weakref.ReferenceType] = None
@@ -259,14 +279,15 @@ class OctopusMessenger(ClientMessenger, ABC):
                        % (msg.type, msg.sender, msg.receiver, get_remote_station(messenger=self), msg.get('traces')))
             return []
         # handshake accepted, redirecting message
-        self.info(msg='redirect msg(type=%d): %s -> %s | from %s, traces: %s'
-                  % (msg.type, msg.sender, msg.receiver, get_remote_station(messenger=self), msg.get('traces')))
-        return self.deliver_message(msg=msg)
+        sig = get_msg_sig(msg=msg)
+        self.info(msg='redirect msg(type=%d, sig=%s): %s -> %s | from %s, traces: %s'
+                  % (msg.type, sig, msg.sender, msg.receiver, get_remote_station(messenger=self), msg.get('traces')))
+        return self._deliver_message(msg=msg)
 
     @abstractmethod
-    def deliver_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
+    def _deliver_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
         """ call octopus to redirect message """
-        pass
+        return []
 
 
 def get_remote_station(messenger: ClientMessenger) -> ID:
@@ -279,7 +300,7 @@ class InnerMessenger(OctopusMessenger):
     """ Messenger for local station """
 
     # Override
-    def deliver_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
+    def _deliver_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
         priority = 0  # NORMAL
         if msg.receiver.is_broadcast:
             priority = 1  # SLOWER
@@ -291,29 +312,30 @@ class OuterMessenger(OctopusMessenger):
     """ Messenger for remote station """
 
     # Override
-    def process_reliable_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
-        if msg.sender == self.local_station:
-            self.debug(msg='cycled message from this station: %s => %s' % (msg.sender, msg.receiver))
-            return []
-        return super().process_reliable_message(msg=msg)
-
-    # Override
-    def deliver_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
+    def _deliver_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
         priority = 0  # NORMAL
         if msg.receiver.is_broadcast:
             priority = 1  # SLOWER
         octopus = self.octopus
         return octopus.income_message(msg=msg, priority=priority)
 
+    # Override
+    def process_reliable_message(self, msg: ReliableMessage) -> List[ReliableMessage]:
+        if msg.sender == self.local_station:
+            self.debug(msg='cycled message from this station: %s => %s' % (msg.sender, msg.receiver))
+            return []
+        return super().process_reliable_message(msg=msg)
+
     # Override
     def handshake_success(self):
         super().handshake_success()
         station = self.session.station
+        update_station(station=station, database=self.octopus.database)
         octopus = self.octopus
         octopus.add_index(identifier=station.identifier, terminal=self.terminal)
 
 
-def create_messenger(facebook: CommonFacebook, database: MessageDBI,
+def create_messenger(facebook: ClientFacebook, database: MessageDBI,
                      session: ClientSession, messenger_class) -> OctopusMessenger:
     assert issubclass(messenger_class, OctopusMessenger), 'messenger class error: %s' % messenger_class
     # 1. create messenger with session and MessageDB
@@ -332,3 +354,18 @@ def create_terminal(messenger: OctopusMessenger) -> Terminal:
     messenger.terminal = terminal
     terminal.start()
     return terminal
+
+
+def update_station(station: Station, database: SessionDBI):
+    Log.info(msg='update station: %s' % station)
+    # SP ID
+    provider = station.provider
+    if provider is None:
+        provider = ProviderInfo.GSP
+    # new info
+    sid = station.identifier
+    host = station.host
+    port = station.port
+    assert not sid.is_broadcast, 'station ID error: %s' % sid
+    assert host is not None and port > 0, 'station error: %s, %d' % (host, port)
+    database.update_station(identifier=sid, host=host, port=port, provider=provider, chosen=0)
