@@ -25,18 +25,17 @@
 
 import getopt
 import sys
+import time
 from typing import Optional, Tuple
 
 from dimsdk import ID, Station
 
-from ..utils import Singleton
-from ..common import CommonFacebook
+from ..utils import Singleton, Config
 from ..common import AccountDBI, MessageDBI, SessionDBI
+from ..common import ProviderInfo
 from ..database import AccountDatabase, MessageDatabase, SessionDatabase
 from ..database import Storage
-from ..client import ClientSession
-
-from ..config import Config
+from ..client import ClientSession, ClientFacebook, ClientArchivist
 
 
 @Singleton
@@ -48,7 +47,7 @@ class GlobalVariable:
         self.adb: Optional[AccountDBI] = None
         self.mdb: Optional[MessageDBI] = None
         self.sdb: Optional[SessionDBI] = None
-        self.facebook: Optional[CommonFacebook] = None
+        self.facebook: Optional[ClientFacebook] = None
 
 
 def show_help(cmd: str, app_name: str, default_config: str):
@@ -111,28 +110,43 @@ def create_database(config: Config) -> Tuple[AccountDBI, MessageDBI, SessionDBI]
     adb.show_info()
     mdb.show_info()
     sdb.show_info()
+    # default provider
+    provider = ProviderInfo.GSP
     # add neighbors
     neighbors = config.neighbors
     for node in neighbors:
-        print('adding neighbor node: (%s:%d), ID=%s' % (node.host, node.port, node.identifier))
-        sdb.add_neighbor(host=node.host, port=node.port)
+        print('adding neighbor node: %s' % node)
+        sdb.add_station(identifier=None, host=node.host, port=node.port, provider=provider)
     return adb, mdb, sdb
 
 
-def create_facebook(database: AccountDBI, current_user: ID) -> CommonFacebook:
+def create_facebook(database: AccountDBI, current_user: ID) -> ClientFacebook:
     """ Step 3: create facebook """
-    facebook = CommonFacebook(database=database)
+    facebook = ClientFacebook()
+    # create archivist for facebook
+    archivist = ClientArchivist(database=database)
+    archivist.facebook = facebook
+    facebook.archivist = archivist
     # make sure private keys exists
     sign_key = facebook.private_key_for_visa_signature(identifier=current_user)
     msg_keys = facebook.private_keys_for_decryption(identifier=current_user)
     assert sign_key is not None, 'failed to get sign key for current user: %s' % current_user
-    assert msg_keys is not None and len(msg_keys) > 0, 'failed to get msg keys: %s' % current_user
+    assert len(msg_keys) > 0, 'failed to get msg keys: %s' % current_user
     print('set current user: %s' % current_user)
-    facebook.current_user = facebook.user(identifier=current_user)
+    user = facebook.user(identifier=current_user)
+    assert user is not None, 'failed to get current user: %s' % current_user
+    visa = user.visa
+    if visa is not None:
+        # refresh visa
+        now = time.time()
+        visa.set_property(key='time', value=now)
+        visa.sign(private_key=sign_key)
+        facebook.save_document(document=visa)
+    facebook.current_user = user
     return facebook
 
 
-def create_session(facebook: CommonFacebook, database: SessionDBI, host: str, port: int) -> ClientSession:
+def create_session(facebook: ClientFacebook, database: SessionDBI, host: str, port: int) -> ClientSession:
     # 1. create station with remote host & port
     station = Station(host=host, port=port)
     station.data_source = facebook
