@@ -29,7 +29,7 @@ import os
 import sys
 import getopt
 
-from mkm import ID
+from dimsdk import ID
 
 path = os.path.abspath(__file__)
 path = os.path.dirname(path)
@@ -37,14 +37,12 @@ path = os.path.dirname(path)
 path = os.path.dirname(path)
 sys.path.insert(0, path)
 
-from dimples.utils import Log
+from dimples.utils import Log, Config
 from dimples.database import Storage
 
-from dimples.config import Config
 from dimples.register.shared import GlobalVariable
 from dimples.register.shared import create_database
-from dimples.register.generate import generate
-from dimples.register.modify import modify
+from dimples.register.shared import generate, modify
 
 
 #
@@ -110,11 +108,11 @@ def main():
     create_database(shared=shared)
     # check actions
     if len(args) == 1 and args[0] == 'generate':
-        generate(db=shared.adb)
+        generate(database=shared.adb)
     elif len(args) == 2 and args[0] == 'modify':
         identifier = ID.parse(identifier=args[1])
         assert identifier is not None, 'ID error: %s' % args[1]
-        modify(identifier=identifier, db=shared.adb)
+        modify(identifier=identifier, database=shared.adb)
     else:
         show_help()
 
