@@ -36,22 +36,25 @@
 """
 
 import socket
+import threading
 import traceback
 from typing import Optional, List, Tuple
 
-from dimsdk import ID
+from dimsdk import ID, EntityType
+from dimsdk import ReliableMessage
 
 from startrek import Docker, DockerStatus
-from startrek import Arrival
+from startrek import Arrival, Departure
 
-from ..utils import hex_encode, random_bytes
 from ..utils import Log
-from ..common import SessionDBI
-from ..database import ReliableMessageTable
+from ..utils import hex_encode, random_bytes
+from ..utils import get_msg_sig, get_msg_info
+from ..common import SessionDBI, MessageDBI, ReliableMessageDBI
+from ..conn import MessageWrapper
 from ..conn import BaseSession
 from ..conn import WSArrival, MarsStreamArrival, MTPStreamArrival
 
-from .push_service import PushCenter
+from .push import PushCenter
 
 
 def generate_session_key() -> str:
@@ -87,19 +90,28 @@ class ServerSession(BaseSession):
     def key(self) -> str:
         return self.__key
 
+    def __load_cached_messages(self):
+        if self.identifier is None or not self.active:
+            return False
+        # load cached message asynchronously
+        threading.Thread(target=load_cached_messages, args=(self,)).start()
+        # load_cached_messages(session=self)
+        return True
+
     # Override
     def set_identifier(self, identifier: ID) -> bool:
         old = self.identifier
         if super().set_identifier(identifier=identifier):
             session_change_id(session=self, new_id=identifier, old_id=old)
-            load_cached_messages(session=self)
+            self.__load_cached_messages()
             return True
 
     # Override
-    def set_active(self, active: bool, when: float = None):
+    def set_active(self, active: bool, when: float = None) -> bool:
         if super().set_active(active=active, when=when):
             session_change_active(session=self, active=active)
-            load_cached_messages(session=self)
+            if active:
+                self.__load_cached_messages()
             return True
 
     @property  # Override
@@ -129,12 +141,14 @@ class ServerSession(BaseSession):
         # super().docker_received(ship=ship, docker=docker)
         all_responses = []
         messenger = self.messenger
+        # 1. get data packages from arrival ship's payload
         packages = get_data_packages(ship=ship)
         for pack in packages:
             try:
+                # 2. process each data package
                 responses = messenger.process_package(data=pack)
                 for res in responses:
-                    if res is None or len(res) == 0:
+                    if len(res) == 0:
                         # should not happen
                         continue
                     all_responses.append(res)
@@ -147,6 +161,7 @@ class ServerSession(BaseSession):
         gate = self.gate
         source = docker.remote_address
         destination = docker.local_address
+        # 3. send responses
         if len(all_responses) > 0:
             # respond separately
             for res in all_responses:
@@ -155,6 +170,16 @@ class ServerSession(BaseSession):
             # station MUST respond something to client request (Tencent Mars)
             gate.send_response(payload=b'', ship=ship, remote=source, local=destination)
 
+    # Override
+    def docker_sent(self, ship: Departure, docker: Docker):
+        if isinstance(ship, MessageWrapper):
+            msg = ship.msg
+            if msg is not None:
+                # remove from database for actual receiver
+                receiver = self.identifier
+                db = self.messenger.database
+                remove_reliable_message(msg=msg, receiver=receiver, database=db)
+
 
 def get_data_packages(ship: Arrival) -> List[bytes]:
     # get payload
@@ -197,26 +222,38 @@ def session_change_active(session: ServerSession, active: bool):
         return True
 
 
-def load_cached_messages(session: ServerSession) -> int:
+def load_cached_messages(session: ServerSession):
     identifier = session.identifier
-    if identifier is None or not session.active:
-        return -1
+    assert identifier is not None and session.active, 'session error: %s' % identifier
     messenger = session.messenger
     db = messenger.database
-    total = 0
-    start = 0
-    limit = 1024
-    remaining = 1
-    while remaining > 0:
-        messages, remaining = db.reliable_messages(receiver=identifier, start=start, limit=limit)
-        start += limit
-        cnt = len(messages)
-        total += cnt
-        Log.info(msg='[DB] %d cached message(s) loaded for: %s' % (cnt, identifier))
-        for msg in messages:
-            data = messenger.serialize_message(msg=msg)
-            session.queue_message_package(msg=msg, data=data, priority=1)
-        if total > ReliableMessageTable.CACHE_LIMIT:
-            Log.warning(msg='[DB] too many cached messages for: %s' % identifier)
-            break
-    return total
+    limit = ReliableMessageDBI.CACHE_LIMIT
+    messages = db.reliable_messages(receiver=identifier, limit=limit)
+    cnt = len(messages)
+    Log.info(msg='[DB] %d cached message(s) loaded for: %s' % (cnt, identifier))
+    for msg in messages:
+        data = messenger.serialize_message(msg=msg)
+        ok = session.queue_message_package(msg=msg, data=data, priority=1)
+        sig = get_msg_sig(msg=msg)
+        Log.info(msg='queue message for: %s, %s, %s' % (identifier, ok, sig))
+
+
+def remove_reliable_message(msg: ReliableMessage, receiver: ID, database: MessageDBI):
+    # 0. if session ID is empty, means user not login;
+    #    this message must be a handshake command, and
+    #    its receiver must be the targeted user.
+    # 1. if this session is a station, check original receiver;
+    #    a message to station won't be stored.
+    # 2. if the msg.receiver is a different user ID, means it's
+    #    a roaming message, remove it for actual receiver.
+    # 3. if the original receiver is a group, it must have been
+    #    replaced to the group assistant ID by GroupDeliver.
+    if receiver is None or receiver.type == EntityType.STATION:
+        # if msg.receiver == receiver:
+        #     # station message won't be stored
+        #     return False
+        receiver = msg.receiver
+    info = get_msg_info(msg=msg)
+    Log.info(msg='message sent for %s, remove from db: %s' % (receiver, info))
+    # remove sent message from database
+    return database.remove_reliable_message(msg=msg, receiver=receiver)
