@@ -25,23 +25,24 @@
 
 import getopt
 import sys
+import time
 from typing import Optional, Tuple
 
 from dimsdk import ID
 
-from ..utils import Singleton
-from ..common import CommonFacebook
+from ..utils import Singleton, Config
+from ..common import AddressNameServer, ANSFactory
+from ..common import CommonFacebook, CommonMessenger
 from ..common import AccountDBI, MessageDBI, SessionDBI
+from ..common import ProviderInfo
 from ..database import AccountDatabase, MessageDatabase, SessionDatabase
 from ..database import Storage
-from ..server import Pusher, DefaultPusher, PushCenter
+from ..server import ServerArchivist
+from ..server import ServerSession
+from ..server import ServerMessenger
+from ..server import ServerMessagePacker
+from ..server import ServerMessageProcessor
 from ..server import Dispatcher
-from ..server import UserDeliver, BotDeliver, StationDeliver
-from ..server import GroupDeliver, BroadcastDeliver
-from ..server import DeliverWorker, DefaultRoamer
-
-from ..config import Config
-from .ans import AddressNameService, AddressNameServer, ANSFactory
 
 
 @Singleton
@@ -54,7 +55,7 @@ class GlobalVariable:
         self.mdb: Optional[MessageDBI] = None
         self.sdb: Optional[SessionDBI] = None
         self.facebook: Optional[CommonFacebook] = None
-        self.pusher: Optional[Pusher] = None
+        self.messenger: Optional[CommonMessenger] = None  # only for archivist
 
 
 def show_help(cmd: str, app_name: str, default_config: str):
@@ -116,73 +117,75 @@ def create_database(config: Config) -> Tuple[AccountDBI, MessageDBI, SessionDBI]
     adb.show_info()
     mdb.show_info()
     sdb.show_info()
+    # default provider
+    provider = ProviderInfo.GSP
     # add neighbors
     neighbors = config.neighbors
     for node in neighbors:
-        print('adding neighbor node: (%s:%d), ID="%s"' % (node.host, node.port, node.identifier))
-        sdb.add_neighbor(host=node.host, port=node.port, identifier=node.identifier)
+        print('adding neighbor node: %s' % node)
+        sdb.add_station(identifier=None, host=node.host, port=node.port, provider=provider)
     return adb, mdb, sdb
 
 
 def create_facebook(database: AccountDBI, current_user: ID) -> CommonFacebook:
     """ Step 3: create facebook """
-    facebook = CommonFacebook(database=database)
+    facebook = CommonFacebook()
+    # create archivist for facebook
+    shared = GlobalVariable()
+    shared.messenger = create_messenger(facebook=facebook, database=shared.mdb, session=None)
+    archivist = ServerArchivist(database=database)
+    archivist.messenger = shared.messenger
+    archivist.facebook = facebook
+    facebook.archivist = archivist
     # make sure private keys exists
     sign_key = facebook.private_key_for_visa_signature(identifier=current_user)
     msg_keys = facebook.private_keys_for_decryption(identifier=current_user)
     assert sign_key is not None, 'failed to get sign key for current user: %s' % current_user
-    assert msg_keys is not None and len(msg_keys) > 0, 'failed to get msg keys: %s' % current_user
+    assert len(msg_keys) > 0, 'failed to get msg keys: %s' % current_user
     print('set current user: %s' % current_user)
-    facebook.current_user = facebook.user(identifier=current_user)
+    user = facebook.user(identifier=current_user)
+    assert user is not None, 'failed to get current user: %s' % current_user
+    visa = user.visa
+    if visa is not None:
+        # refresh visa
+        now = time.time()
+        visa.set_property(key='time', value=now)
+        visa.sign(private_key=sign_key)
+        facebook.save_document(document=visa)
+    facebook.current_user = user
     return facebook
 
 
-def create_ans(config: Config) -> AddressNameService:
-    """ Step 4: create ANS """
+def create_messenger(facebook: CommonFacebook, database: MessageDBI,
+                     session: Optional[ServerSession]) -> ServerMessenger:
+    # 1. create messenger with session and MessageDB
+    messenger = ServerMessenger(session=session, facebook=facebook, database=database)
+    # 2. create packer, processor, filter for messenger
+    #    they have weak references to session, facebook & messenger
+    messenger.packer = ServerMessagePacker(facebook=facebook, messenger=messenger)
+    messenger.processor = ServerMessageProcessor(facebook=facebook, messenger=messenger)
+    # 3. set weak reference messenger in session
+    if session is not None:
+        session.messenger = messenger
+    return messenger
+
+
+def create_dispatcher(shared: GlobalVariable) -> Dispatcher:
+    """ Step 4: create dispatcher """
+    dispatcher = Dispatcher()
+    dispatcher.mdb = shared.mdb
+    dispatcher.sdb = shared.sdb
+    dispatcher.facebook = shared.facebook
+    return dispatcher
+
+
+def create_ans(config: Config) -> AddressNameServer:
+    """ Step 5: create ANS """
     ans = AddressNameServer()
     factory = ID.factory()
     ID.register(factory=ANSFactory(factory=factory, ans=ans))
     # load ANS records from 'config.ini'
-    ans.fix(fixed=config.ans_records)
+    ans_records = config.ans_records
+    if ans_records is not None:
+        ans.fix(records=ans_records)
     return ans
-
-
-def create_pusher(shared: GlobalVariable) -> Pusher:
-    """ Step 5: create pusher """
-    pusher = DefaultPusher(facebook=shared.facebook)
-    shared.pusher = pusher
-    # start PushCenter
-    center = PushCenter()
-    # TODO: add push services
-    center.start()
-    return pusher
-
-
-def create_dispatcher(shared: GlobalVariable) -> Dispatcher:
-    """ Step 6: create dispatcher """
-    dispatcher = Dispatcher()
-    dispatcher.database = shared.mdb
-    dispatcher.facebook = shared.facebook
-    # set base deliver delegates
-    user_deliver = UserDeliver(database=shared.mdb, pusher=shared.pusher)
-    bot_deliver = BotDeliver(database=shared.mdb)
-    station_deliver = StationDeliver()
-    dispatcher.set_user_deliver(deliver=user_deliver)
-    dispatcher.set_bot_deliver(deliver=bot_deliver)
-    dispatcher.set_station_deliver(deliver=station_deliver)
-    # set special deliver delegates
-    group_deliver = GroupDeliver(facebook=shared.facebook)
-    broadcast_deliver = BroadcastDeliver(database=shared.sdb)
-    dispatcher.set_group_deliver(deliver=group_deliver)
-    dispatcher.set_broadcast_deliver(deliver=broadcast_deliver)
-    # set roamer & worker
-    roamer = DefaultRoamer(database=shared.mdb)
-    worker = DeliverWorker(database=shared.sdb, facebook=shared.facebook)
-    dispatcher.set_roamer(roamer=roamer)
-    dispatcher.set_deliver_worker(worker=worker)
-    # start all delegates
-    user_deliver.start()
-    bot_deliver.start()
-    station_deliver.start()
-    roamer.start()
-    return dispatcher
